-- vytvoen tabulek
CREATE TABLE Clen (
  IDClen smallint identity (1,1),
  Jmeno varchar(15) NOT NULL,
  Prijmeni varchar(20) NOT NULL,
  RodCislo char(11) NOT NULL,
  Ulice varchar(40) NOT NULL,
  CP varchar(6) NOT NULL,
  Mesto varchar(40) NOT NULL,
  PSC char(5) NOT NULL,
  Telefon varchar(14) NOT NULL,
  Email varchar(60),
  DatNar date NOT NULL,
  Pohlavi bit NOT NULL,
  SUdaje bit NOT NULL DEFAULT '0',
  SOddil bit NOT NULL DEFAULT '0',
  SInfo bit NOT NULL DEFAULT '0',
  SFoto bit NOT NULL DEFAULT '0',
  DatReGOdd date NOT NULL,
  DatRegSvaz date,
  Plavec bit  NOT NULL DEFAULT '0',
  PRIMARY KEY (IDClen))   
GO

CREATE TABLE Trener (
  IDTrener smallint CHECK (IDTrener >= 0),
  CisloUctu char(22) NOT NULL,
  Odmena money DEFAULT 0 CHECK (Odmena >= 0),
  IDClen smallint CHECK (IDClen > 0),
  PRIMARY KEY (IDTrener),
  FOREIGN KEY (IDClen) REFERENCES Clen (IDClen) ON UPDATE CASCADE)
GO

CREATE TABLE Ucel (
  VSKod tinyint CHECK (VSKod > 0 AND VSKod <= 99),
  Ucel varchar(255) NOT NULL,
  PRIMARY KEY (VSKod))
GO

CREATE TABLE Platba (
  IDPlatba int identity(1,1),
  VSOdd tinyint DEFAULT '16' NOT NULL CHECK (VSOdd >= 0 AND VSOdd <= 99),
  Castka money NOT NULL CHECK (Castka >= 0),
  DatumUhr date,
  IDClen smallint CHECK (IDClen > 0),
  VSKod tinyint CHECK (VSKod > 0 AND VSKod <= 99),
  PRIMARY KEY (IDPlatba),
  FOREIGN KEY (IDClen) REFERENCES Clen (IDClen) ON UPDATE CASCADE,
  FOREIGN KEY (VSKod) REFERENCES Ucel (VSKod) ON UPDATE CASCADE)
GO

CREATE TABLE Kvalifikace (
  IDKvalifikace varchar(5),
  Nazev varchar(100) NOT NULL,
  PRIMARY KEY (IDKvalifikace))
GO

CREATE TABLE TrenerskaKvalifikace (
  Platnost date,
  IDKvalifikace varchar(5),
  IDTrener smallint CHECK (IDTrener > 0),
  PRIMARY KEY (IDKvalifikace, IDTrener),
  FOREIGN KEY (IDKvalifikace) REFERENCES Kvalifikace (IDKvalifikace) ON UPDATE CASCADE ON DELETE CASCADE,
  FOREIGN KEY (IDTrener) REFERENCES Trener (IDTrener) ON UPDATE CASCADE ON DELETE CASCADE)
GO

CREATE TABLE Dodavatel (
  IDDodavatel smallint identity(1,1),
  Nazev varchar(30) NOT NULL,
  Telefon varchar(14),
  Email varchar(60),
  Web varchar(50),
  PRIMARY KEY (IDDodavatel))
GO

CREATE TABLE Pomucka (
  IDPomucka char(6),
  Nazev varchar(20) NOT NULL,
  Popis varchar(255),
  Barva varchar(10),
  Pocet smallint NOT NULL DEFAULT '1' CHECK (Pocet >= 0),
  DatumNakupu date,
  Cena money CHECK (Cena >= 0),
  IDDodavatel smallint CHECK (IDDodavatel > 0),
  PRIMARY KEY (IDPomucka),
  FOREIGN KEY (IDDodavatel) REFERENCES Dodavatel (IDDodavatel) ON UPDATE CASCADE)
GO

CREATE TABLE ZaznamOPoskozeni (
  IDZaznam smallint identity(1,1),
  Nazev varchar(30) NOT NULL,
  Datum datetime NOT NULL,
  Popis text,
  IDTrener smallint CHECK (IDTrener > 0),
  IDPomucka char(6),
  PRIMARY KEY (IDZaznam),
  FOREIGN KEY (IDTrener) REFERENCES Trener (IDTrener) ON UPDATE CASCADE,
  FOREIGN KEY (IDPomucka) REFERENCES Pomucka (IDPomucka) ON UPDATE CASCADE)
GO

CREATE TABLE AktivitaPopis (
  Kod varchar(5),
  Nazev varchar(50) NOT NULL,
  Popis varchar(255),
  PRIMARY KEY (Kod))
GO

CREATE TABLE Aktivita (
  IDAktivita smallint identity(1,1),
  Kod varchar(5),
  Zacatek datetime NOT NULL,
  Konec datetime,
  PRIMARY KEY (IDAktivita),
  FOREIGN KEY (Kod) REFERENCES AktivitaPopis (Kod) ON UPDATE CASCADE)
GO

CREATE TABLE DochazkaAktivita (
  IDAktivita smallint CHECK (IDAktivita > 0),
  IDClen smallint CHECK (IDClen > 0),
  PRIMARY KEY (IDAktivita, IDClen),
  FOREIGN KEY (IDAktivita) REFERENCES Aktivita (IDAktivita) ON UPDATE CASCADE,
  FOREIGN KEY (IDClen) REFERENCES Clen (IDClen) ON UPDATE CASCADE)
GO

CREATE TABLE Disciplina (
  DisciplinaZkratka varchar(10),
  Vzdalenost smallint NOT NULL CHECK (Vzdalenost > 0),
  Zpusob varchar(50) NOT NULL,
  PRIMARY KEY (DisciplinaZkratka))
GO

CREATE TABLE Zavod (
  IDZavod smallint identity(1,1),
  Nazev varchar(100) NOT NULL,
  Mesto varchar(40) NOT NULL,
  Bazen varchar(40),
  DatumOd date NOT NULL,
  DatumDo date,
  DelkaBazenu smallint NOT NULL CHECK (DelkaBazenu > 0),
  TypMereni varchar(30) NOT NULL,
  PRIMARY KEY (IDZavod))
GO

CREATE TABLE UcastnikZavodu (
  IDZavod smallint CHECK (IDZavod > 0),
  IDClen smallint CHECK (IDClen > 0),
  PRIMARY KEY (IDZavod, IDClen),
  FOREIGN KEY (IDZavod) REFERENCES Zavod (IDZavod) ON UPDATE CASCADE,
  FOREIGN KEY (IDClen) REFERENCES Clen (IDClen) ON UPDATE CASCADE)
GO

CREATE TABLE Vysledek (
  IDVysledek int identity(1,1),
  DisciplinaZkratka varchar(10) NOT NULL,
  CasCil time(2) NOT NULL,
  Umisteni smallint CHECK (Umisteni > 0),
  Cas1 time(2),
  Datum date,
  IDZavod smallint CHECK (IDZavod > 0),
  PRIMARY KEY(IDVysledek),
  FOREIGN KEY (DisciplinaZkratka) REFERENCES Disciplina (DisciplinaZkratka) ON UPDATE CASCADE,
  FOREIGN KEY (IDZavod) REFERENCES Zavod (IDZavod) ON UPDATE CASCADE)
GO

CREATE TABLE VysledekClen (
  IDVysledek int CHECK (IDVysledek > 0),
  IDClen smallint CHECK (IDClen > 0),
  Poradi tinyint NOT NULL CHECK (Poradi > 0 AND Poradi <= 4),
  PRIMARY KEY (IDVysledek, IDClen),
  FOREIGN KEY (IDVysledek) REFERENCES Vysledek (IDVysledek) ON UPDATE CASCADE ON DELETE CASCADE,
  FOREIGN KEY (IDClen) REFERENCES Clen (IDClen) ON UPDATE CASCADE)
GO

CREATE TABLE ZaznamOZraneni (
  IDZaznam smallint identity(1,1),
  DatumCas datetime NOT NULL,
  Popis text NOT NULL,
  PRIMARY KEY (IDZaznam)) 
GO

CREATE TABLE ClenZaznam (
  IDZaznam smallint CHECK (IDZaznam > 0),
  IDClen smallint CHECK (IDClen > 0),
  Zraneny bit NOT NULL,
  PRIMARY KEY (IDZaznam, IDClen),
  FOREIGN KEY (IDZaznam) REFERENCES ZaznamOZraneni (IDZaznam) ON UPDATE CASCADE,
  FOREIGN KEY (IDClen) REFERENCES Clen (IDClen) ON UPDATE CASCADE)
GO

CREATE TABLE ClenskaKarta (
  IDKarta smallint CHECK (IDKarta > 0),
  Platnost date NOT NULL,
  IDClen smallint CHECK (IDClen > 0),
  PRIMARY KEY (IDKarta),
  FOREIGN KEY (IDClen) REFERENCES Clen (IDClen) ON UPDATE CASCADE ON DELETE CASCADE)
GO

CREATE TABLE TreninkovaSkupina (
  IDSkupina char(6),
  Rok smallint NOT NULL CHECK (Rok > 0),
  Pololeti bit NOT NULL, 
  Den char(2) NOT NULL, 
  Nazev varchar(30) NOT NULL,
  Popis text,
  CasZac smalldatetime NOT NULL,
  CasKon smalldatetime NOT NULL,
  PRIMARY KEY (IDSkupina))
GO

CREATE TABLE Prislusnost (
  IDClen smallint CHECK (IDClen > 0),
  IDSkupina char(6),
  PRIMARY KEY (IDClen, IDSkupina),
  FOREIGN KEY (IDClen) REFERENCES Clen (IDClen) ON UPDATE CASCADE ON DELETE CASCADE,
  FOREIGN KEY (IDSkupina) REFERENCES TreninkovaSkupina (IDSkupina) ON UPDATE CASCADE)
GO

CREATE TABLE Trenink (
  Datum date,
  IDSkupina char(6),
  PRIMARY KEY (Datum, IDSkupina),
  FOREIGN KEY (IDSkupina) REFERENCES TreninkovaSkupina (IDSkupina) ON UPDATE CASCADE)
GO 

CREATE TABLE DochazkaTrenink (
  IDClen smallint CHECK (IDClen > 0),
  IDSkupina char(6),
  Datum date,
  Ucast char(1) NOT NULL,
  PRIMARY KEY (IDClen, IDSkupina, Datum),
  FOREIGN KEY (Datum, IDSkupina) REFERENCES Trenink (Datum, IDSkupina) ON UPDATE CASCADE,
  FOREIGN KEY (IDClen) REFERENCES Clen (IDClen) ON UPDATE CASCADE)
GO

CREATE TABLE CastTreninku (
  IDSkupina char(6),
  Datum date,
  Poradi smallint CHECK (Poradi > 0),
  Opakovani smallint NOT NULL DEFAULT '1' CHECK (Opakovani > 0),
  Delka smallint CHECK (Delka > 0),
  Zpusob char(2),
  NohyRuce char(1),
  TC bit NOT NULL DEFAULT '0',
  Ploutve bit NOT NULL DEFAULT '0',
  Packy bit NOT NULL DEFAULT '0',
  RozpVypl char(4),
  Poznamka varchar(50),
  PRIMARY KEY (IDSkupina, Datum, Poradi),
  FOREIGN KEY (Datum, IDSkupina) REFERENCES Trenink (Datum, IDSkupina) ON UPDATE CASCADE ON DELETE CASCADE)
GO

-- naplnn tabulek daty
INSERT INTO Clen (Jmeno,Prijmeni,RodCislo,Ulice,CP,Mesto,PSC,Telefon,Email,DatNar,Pohlavi,SUdaje,SOddil,SInfo,SFoto,DatReGOdd,DatRegSvaz,Plavec)
VALUES ('Adam','Novk','890606/9878','Dlouh','123','Brno','61200','+420777123456','novak@email.cz','1989-06-06','1','1','1','1','1','2012-01-23',NULL,'1')
INSERT INTO Clen (Jmeno,Prijmeni,RodCislo,Ulice,CP,Mesto,PSC,Telefon,Email,DatNar,Pohlavi,SUdaje,SOddil,SInfo,SFoto,DatReGOdd,DatRegSvaz,Plavec)
VALUES ('Adla','Novkov','895606/9878','Dlouh','123','Brno','61200','+420777123456','novakova@email.cz','1989-06-06','0','1','1','1','1','2012-01-23',NULL,'1')
INSERT INTO Clen (Jmeno,Prijmeni,RodCislo,Ulice,CP,Mesto,PSC,Telefon,Email,DatNar,Pohlavi,SUdaje,SOddil,SInfo,SFoto,DatReGOdd,DatRegSvaz,Plavec)
VALUES ('Karel','Mal','820306/1878','Milady Horkov','13','Krnov','65202','+420603123456','maly@email.cz','1982-03-06','1','1','1','1','1','2010-01-23',NULL,'1')
GO

SELECT * FROM Clen
GO

INSERT INTO Trener Values ('0001','000000-1347894567/0300','120','3')
GO

SELECT * FROM Trener
GO

INSERT INTO ClenskaKarta VALUES ('1','2013-12-31','3')
INSERT INTO ClenskaKarta VALUES ('2','2013-12-31','2')
INSERT INTO ClenskaKarta VALUES ('3','2013-12-31','1')
GO

SELECT * FROM ClenskaKarta
GO

INSERT INTO TreninkovaSkupina VALUES ('1312N0','2013','0','t','Neplavci','Skupina urcena neplavcum','17:00','17:30')
GO

SELECT * FROM TreninkovaSkupina
GO

INSERT INTO Prislusnost VALUES ('1','1312N0')
INSERT INTO Prislusnost VALUES ('2','1312N0')
INSERT INTO Prislusnost VALUES ('3','1312N0')
GO

SELECT * FROM Prislusnost
GO

INSERT INTO Trenink VALUES ('2013-05-14','1312N0')
INSERT INTO Trenink VALUES ('2013-05-07','1312N0')
INSERT INTO Trenink VALUES ('2013-04-07','1312N0')  
GO

SELECT * FROM Trenink
GO

INSERT INTO DochazkaTrenink (IDClen,IDSkupina,Datum,Ucast) VALUES ('1','1312N0','2013-05-07','U') 
INSERT INTO DochazkaTrenink (IDClen,IDSkupina,Datum,Ucast) VALUES ('2','1312N0','2013-05-07','U') 
INSERT INTO DochazkaTrenink (IDClen,IDSkupina,Datum,Ucast) VALUES ('3','1312N0','2013-05-07','U') 
INSERT INTO DochazkaTrenink (IDClen,IDSkupina,Datum,Ucast) VALUES ('1','1312N0','2013-05-14','U') 
INSERT INTO DochazkaTrenink (IDClen,IDSkupina,Datum,Ucast) VALUES ('3','1312N0','2013-05-14','U')
GO

SELECT * FROM DochazkaTrenink
GO

INSERT INTO CastTreninku (IDSkupina,Datum,Poradi,Delka,RozpVypl) VALUES ('1312N0','2013-05-07','1','100','rozp')
INSERT INTO CastTreninku (IDSkupina,Datum,Poradi,Opakovani,Delka,Zpusob,NohyRuce,Ploutve) VALUES ('1312N0','2013-05-07','2','16','50','VZ','N','1')
INSERT INTO CastTreninku (IDSkupina,Datum,Poradi,Delka,RozpVypl) VALUES ('1312N0','2013-05-07','3','100','vypl')
INSERT INTO CastTreninku (IDSkupina,Datum,Poradi,Delka,RozpVypl) VALUES ('1312N0','2013-05-14','1','100','rozp')
INSERT INTO CastTreninku (IDSkupina,Datum,Poradi,Opakovani,Delka,Zpusob) VALUES ('1312N0','2013-05-14','2','16','100','P')
INSERT INTO CastTreninku (IDSkupina,Datum,Poradi,Delka,RozpVypl) VALUES ('1312N0','2013-05-14','3','100','vypl')
INSERT INTO CastTreninku (IDSkupina,Datum,Poradi,Delka,RozpVypl) VALUES ('1312N0','2013-04-07','1','100','rozp')
GO

SELECT * FROM CastTreninku
GO

-- vytvoen procedur
CREATE PROCEDURE VlozClena
@Jmeno varchar(15),
@Prijmeni varchar(20),
@RC char(11),
@Ulice varchar(40),
@CP varchar(6),
@Mesto varchar(40),
@PSC char(5),
@Telefon varchar(14),
@Email varchar(60) = NULL,
@DatNar date,
@Pohlavi bit,
@Sudaje bit,
@Soddil bit,
@SInfo bit,
@SFoto bit,
@DatReGOdd date,
@DatRegSvaz date = NULL,
@Plavec bit
AS
BEGIN
  DECLARE @Cislo int 
  SELECT @Cislo = IDClen FROM Clen WHERE RodCislo = @RC
  IF (@Cislo is NULL)
    BEGIN
      INSERT INTO Clen (Jmeno,Prijmeni,RodCislo,Ulice,CP,Mesto,PSC,
                        Telefon,Email,DatNar,Pohlavi,SUdaje,SOddil, 
                        SInfo,SFoto,DatReGOdd,DatRegSvaz,Plavec)
      values (@Jmeno,@Prijmeni,@RC,@Ulice,@CP,@Mesto,@PSC,@Telefon,@Email,@DatNar,
@Pohlavi,@Sudaje,@Soddil,@SInfo,@SFoto,@DatReGOdd,@DatRegSvaz,@Plavec)
      SET @Cislo = IDENT_CURRENT('Clen')
      PRINT 'Osoba byla ulozena. Jeji cislo je: ' + CAST (@Cislo AS varchar)           
    END
  ELSE
    BEGIN    
     PRINT 'Osoba se jiz nachazi v databazi! Jeji cislo je: ' + CAST (@Cislo AS varchar)
    END
END
GO

CREATE PROCEDURE CelkemZaObdobi
@IDSkupina char(6),
@DatOd date,
@DatDo date
AS
BEGIN
  DECLARE @ID char(6)
  SELECT @ID = IDSkupina FROM TreninkovaSkupina WHERE IDSkupina = @IDSkupina
  IF (@ID is NULL)
  BEGIN 
    PRINT 'Zadana treninkova skupina neexistuje!'
  END
  ELSE IF (@DatOd > @DatDo)
  BEGIN
    PRINT 'Zkontrolujte datum!'
  END
  ELSE
    BEGIN
      SELECT IDSkupina, SUM(Delka*Opakovani) AS Celkem FROM CastTreninku WHERE Datum >= @DatOd AND Datum <= @DatDo 
AND IDSkupina = @IDSkupina GROUP BY (IDSkupina)
    END
END
GO

CREATE PROCEDURE CelkemZaObdobiClen
@IDClen smallint,
@DatOd date,
@DatDo date
AS
BEGIN
  DECLARE @ID smallint
  SELECT @ID = IDClen FROM Clen WHERE IDClen = @IDClen
  IF (@ID is NULL)
    BEGIN 
      PRINT 'Zadany clen nenalezen v systemu!'
    END
  ELSE IF (@DatOd > @DatDo)
    BEGIN
      PRINT 'Zkontrolujte datum!'
    END
  ELSE
    BEGIN
     DECLARE @Jmeno varchar(15)
     DECLARE @Prijmeni varchar(20)
     DECLARE @Vysledek int
     SELECT @Vysledek = SUM(t.Delka*t.Opakovani) FROM DochazkaTrenink c, CastTreninku t WHERE t.Datum >= @DatOd AND t.Datum <= @DatDo AND c.IDClen = @IDClen AND c.Ucast = 'U' GROUP BY (c.IDClen)
     SELECT @Jmeno = Clen.Jmeno FROM Clen WHERE Clen.IDClen = @IDClen
     SELECT @Prijmeni = Clen.Prijmeni FROM Clen WHERE Clen.IDClen = @IDClen
     PRINT @Jmeno + ' ' + @Prijmeni + ' uplaval/a v zadanem obdobi ' + CAST (@Vysledek AS varchar) + ' metru.'
  END
END
GO

-- voln procedur
EXECUTE VlozClena 'Filip','Moudr','870203/3312','Tich','78','Znojmo','63100','+420605723845','moudry@email.cz','1987-02-03','1','1','1','1','1','2013-02-12',NULL,'1'
GO

EXECUTE CelkemZaObdobi '1312N0','2013-05-01','2013-05-31'
GO

EXECUTE CelkemZaObdobiClen '1','2013-05-01','2013-05-31'
GO